/*
==============================证件照后期处理接口==============================
*/

package com.WxArrangeApi;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class ArrangeApi {

    private final String MattingAPIServer = "https://idphotox.market.alicloudapi.com";
    private final String MattingAPIUri = "/idphoto/arrange";

    private String httpPost(String httpUrl, Map<String, String> headers, String postData) {

        HttpURLConnection connection = null;
        InputStream is = null;
        OutputStream os = null;
        BufferedReader br = null;
        String result = null;
        try {
            URL url = new URL(httpUrl);

            connection = (HttpURLConnection) url.openConnection();
            if (url.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) connection;
                https.setHostnameVerifier(DO_NOT_VERIFY);
                //con = https;
            }

            connection.setRequestMethod("POST");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);

            connection.setDoOutput(true);

            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }

            os = connection.getOutputStream();
            os.write(postData.getBytes());

            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                StringBuffer sbf = new StringBuffer();
                String temp = null;
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\r\n");
                }
                result = sbf.toString();

            } else {
                System.out.println("bad status code:" + connection.getResponseCode());
                Map<String, List<String>> map = connection.getHeaderFields();
                for (Map.Entry<String, List<String>> entry : map.entrySet()) {
                    System.out.println("Key : " + entry.getKey() +
                            " ,Value : " + entry.getValue());
                }
                is = connection.getErrorStream();
                if (is != null) {
                    br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                    StringBuffer sbf = new StringBuffer();
                    String temp = null;
                    while ((temp = br.readLine()) != null) {
                        sbf.append(temp);
                        sbf.append("\r\n");
                    }
                    result = sbf.toString();
                }

            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != os) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 断开与远程地址url的连接
            connection.disconnect();
        }
        return result;
    }


    final static HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {

        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };


    private String extractValueFromJsonString(String jsonString, String key) {
        String regex = "\"" + key + "\":([^(,|\\}|\\])]*)";
        Matcher matcher = Pattern.compile(regex).matcher(jsonString);
        while (matcher.find()) {
            String val = matcher.group(1);
            val = val.trim();
            if (val.startsWith("\"")) {
                val = val.substring(1);
            }
            if (val.endsWith("\"")) {
                val = val.substring(0, val.length() - 1);
            }
            return val;
        }
        return null;

    }

    private String apiCall(String apiServer, String apiUri, String Appcode, String PhotoKey) {
        try {

            String body = "";

            String accept = "application/json";
            String contentType = "application/octet-stream; charset=utf-8";

            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Accept", accept);
            headers.put("Authorization", "APPCODE " + Appcode);
            headers.put("Content-Type", contentType);


            String result = httpPost(apiServer + apiUri + "?photo_key=" + PhotoKey, headers, body);
            if (result == null) {
                System.out.println("httpPost call fail");
                return null;
            }

            System.out.println("result:" + result);
            return result;


        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    // 证件照排版
    public Boolean ArrangeDetect(String Appcode, String PhotoKey) {
        System.out.println("证件照后期处理接口 ...");
        try {

            String result = apiCall(this.MattingAPIServer,
                    this.MattingAPIUri,
                    Appcode,
                    PhotoKey
            );
            if (result == null) {
                return false;
            }


            // 正确做法是使用json解析库，
            // 这里为了减少依赖，直接使用正则表达式解析
            String status = extractValueFromJsonString(result, "status");
            if (!status.equals("0")) {
                System.out.println("bad status:" + status);
                return false;
            }


            return true;
        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }
        return false;
    }


    public static void main(String[] args) {

        /*
         说明：
             此代码是一份调用接口调用示例，
             WXAIPhotoAPI也只是一份调用示例的class，
             并不是接口本身，
             请不要把WXAIPhotoAPI的参数，当成接口本身的参数
        */
        ArrangeApi api_test = new ArrangeApi();


        String Appcode = "你的appcode";
        String PhotoKey = ""; //photo_key为从“证件照制作接口”或者“更换背景接口”返回的

        api_test.ArrangeDetect(Appcode, PhotoKey);

        return;
    }
}

